﻿using System;
using System.Collections.Concurrent;
using System.Threading.Tasks;


namespace BlockingCollecionTrying
{
    /// <summary>
    /// Adapted from
    /// http://geekswithblogs.net/BlackRabbitCoder/archive/2011/03/03/c.net-little-wonders-concurrentbag-and-blockingcollection.aspx
    /// </summary>
    class Program
    {
        public static void ProduceItems(BlockingCollection<int> buffer, int numToProduce)
        {
            for (int i = 0; i < numToProduce; i++)
            {
                buffer.Add(i);
                Console.WriteLine("Adding item : " + i);
            }
            // once done producing, call CompleteAdding()
            Console.WriteLine("Adding is completed.");
            buffer.CompleteAdding();
        }

        public static void ConsumeItems(String name, BlockingCollection<int> buffer)
        {
            // This will only be true if CompleteAdding() was called AND the buffer is empty.
            while (!buffer.IsCompleted)
            {
                try
                {
                    int item = buffer.Take();
                    Console.WriteLine(name + " Consuming item {0}.", item);
                }
                catch (InvalidOperationException ex)
                {
                    // ignored
                    //http://msdn.microsoft.com/en-us/library/dd997371(v=vs.110).aspx
                }
            }
        }
        static void Main(string[] args)
        {
            BlockingCollection<int> buffer = new BlockingCollection<int>(5);        
            Parallel.Invoke(
                () => ProduceItems(buffer, 20),
                () => ConsumeItems("TWO", buffer),
                () => ConsumeItems("ONE", buffer));
        }
    }
}
